# KeySet takes a list of active key objects and supplies a method to find
# a key given it's ID.
#
# Key objects should be hashes with at least the following structure:
# {
#   'kid': 'ID',
#   'n': 'modulus',
#   'e': 'exponent',
# }
class KeySet
  def initialize(keys:)
    @keys = keys
  end

  def find(key_id)
    key = find_key_or_raise_error(key_id)

    generate_public_key(
      modulus: decode_and_convert_to_number(key['n']),
      exponent: decode_and_convert_to_number(key['e']),
    )
  end

  private

  attr_reader :keys

  def find_key_or_raise_error(id)
    key = keys.find { |k| k['kid'] == id }

    if key.nil? || key.empty?
      fail JWT::VerificationError, 'No keys from key endpoint match the id token'
    end

    key
  end

  def decode_and_convert_to_number(str)
    decoded = JWT.base64url_decode(str)

    decoded.unpack("C*").inject(0) { |sum, (byte, _)| sum * 256 + byte }
  end

  def generate_public_key(modulus:, exponent:)
    seq = OpenSSL::ASN1::Sequence.new([
      OpenSSL::ASN1::Integer.new(modulus),
      OpenSSL::ASN1::Integer.new(exponent),
    ])

    OpenSSL::PKey::RSA.new(seq.to_der)
  end
end
