require 'jwt'
require 'omniauth'

module OmniAuth
  module Strategies
    class AzureADV2
      include OmniAuth::Strategy
      include Omniauth::AzureADV2

      class OAuthError < StandardError; end

      DEFAULT_RESPONSE_TYPE = 'code id_token'
      DEFAULT_RESPONSE_MODE = 'form_post'

      option :name, :azure_adv2

      args [:client_id, :client_secret]

      option :client_id, nil
      option :client_secret, nil

      uid { @claims['sub'] }

      info do
        {
          name: @claims['name'],
          email: @claims['email'].present? ? @claims['email'] : fetch_email,
        }
      end

      credentials { { token: @access_token } }

      extra do
        {
          session_state: @session_state,
          raw_info: {
            id_token: @id_token,
            id_token_claims: @claims,
            id_token_header: @header
          }
        }
      end

      def request_phase
        uri = URI(openid_config.authorization_endpoint)
        uri.query = URI.encode_www_form(client_id: client_id,
                                        redirect_uri: callback_url,
                                        response_mode: response_mode,
                                        response_type: response_type,
                                        scope: 'openid profile email https://graph.microsoft.com/user.read',
                                        nonce: new_nonce)

        redirect uri.to_s
      end

      def callback_phase
        error = request.params['error_reason'] || request.params['error']
        fail(OAuthError, error) if error

        @session_state = request.params['session_state']
        @code = request.params['code']

        @access_token, @id_token = exchange_code_for_creds(@code)
        @claims, @header = validate_and_parse_id_token(@id_token)

        super
      end

      private

      def exchange_code_for_creds(code)
        uri = URI('https://login.microsoftonline.com/common/oauth2/v2.0/token')
        res = Net::HTTP.post_form(
          uri,
          'code': code,
          'grant_type': 'authorization_code',
          'scope': 'https://graph.microsoft.com/user.read',
          'redirect_uri': callback_url,
          'client_id': client_id,
          'client_secret': client_secret,
        )

        parsed = JSON.parse(res.body)

        [parsed['access_token'], parsed['id_token']]
      end

      def fetch_email
        uri = URI('https://graph.microsoft.com/beta/me')
        req = Net::HTTP::Get.new(uri.path)
        req.add_field('Authorization', "Bearer #{@access_token}")

        res = Net::HTTP.start(uri.host, uri.port, use_ssl: true) do |http|
            http.request(req)
        end

        parsed = JSON.parse(res.body)

        parsed['mail'].present? ? parsed['mail'] : parsed['userPrincipalName']
      end

      def new_nonce
        session['omniauth-azure-activedirectory.nonce'] = SecureRandom.uuid
      end

      def openid_config
        OpenIDConfig.fetch(tid: 'common')
      end

      def response_type
        options[:response_type] || DEFAULT_RESPONSE_TYPE
      end

      def response_mode
        options[:response_mode] || DEFAULT_RESPONSE_MODE
      end

      def validate_and_parse_id_token(id_token)
        IDTokenDecoder.new(
          id_token: id_token,
          client_id: client_id,
          nonce: read_nonce,
          keyset: openid_config.keys,
        ).run
      end

      def client_id
        return options.client_id if options.client_id
        fail StandardError, 'No client_id specified in AzureAD configuration.'
      end

      def client_secret
        return options.client_secret if options.client_secret
        fail StandardError, 'No client_secret specified in AzureAD configuration.'
      end

      def read_nonce
        session.delete('omniauth-azure-activedirectory.nonce')
      end
    end
  end
end

OmniAuth.config.add_camelization 'azure_adv2', 'AzureADV2'
